/*
 * Copyright (c) 2010-2020 Belledonne Communications SARL.
 *
 * This file is part of linphone-desktop
 * (see https://www.linphone.org).
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef APP_CONTROLLER_H_
#define APP_CONTROLLER_H_

#include "App.hpp"

// =============================================================================

class AppController {
public:
  AppController (int &argc, char *argv[]);
  ~AppController ();

  App *getApp () const {
    Q_CHECK_PTR(mApp);
    return mApp;
  }

private:
  App *mApp = nullptr;
};

#endif // APP_CONTROLLER_H_
