<?xml version="1.0" encoding="UTF-8"?>
<config xmlns="http://www.linphone.org/xsds/lpconfig.xsd" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.linphone.org/xsds/lpconfig.xsd lpconfig.xsd">
  <section name="proxy_default_values">
    <entry name="avpf" overwrite="true">0</entry>
    <entry name="dial_escape_plus" overwrite="true">0</entry>
    <entry name="publish" overwrite="true">0</entry>
    <entry name="quality_reporting_collector" overwrite="true"></entry>
    <entry name="quality_reporting_enabled" overwrite="true">0</entry>
    <entry name="quality_reporting_interval" overwrite="true">0</entry>
    <entry name="reg_expires" overwrite="true">3600</entry>
    <entry name="reg_identity" overwrite="true"></entry>
    <entry name="reg_proxy" overwrite="true"></entry>
    <entry name="reg_route" overwrite="true"></entry>
    <entry name="reg_sendregister" overwrite="true">1</entry>
    <entry name="nat_policy_ref" overwrite="true"></entry>
    <entry name="refkey" overwrite="true"></entry>
    <entry name="realm" overwrite="true"></entry>
    <entry name="conference_factory_uri" overwrite="true"></entry>
  </section>
  <section name="nat_policy_default_values">
    <entry name="stun_server" overwrite="true"></entry>
    <entry name="protocols" overwrite="true"></entry>
  </section>
  <section name="sip">
    <entry name="use_rls_presence" overwrite="true"></entry>
    <entry name="rls_uri" overwrite="true"></entry>
  </section>
  <section name="assistant">
    <entry name="domain" overwrite="true"></entry>
    <entry name="algorithm" overwrite="true">MD5</entry>
    <entry name="password_max_length" overwrite="true">-1</entry>
    <entry name="password_min_length" overwrite="true">0</entry>
    <entry name="username_length" overwrite="true">-1</entry>
    <entry name="username_max_length" overwrite="true">128</entry>
    <entry name="username_min_length" overwrite="true">1</entry>
    <entry name="username_regex" overwrite="true">^[a-zA-Z0-9+_.\-]*$</entry>
  </section>
</config>
