<?xml version="1.0" encoding="UTF-8"?>
<config xmlns="http://www.linphone.org/xsds/lpconfig.xsd" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.linphone.org/xsds/lpconfig.xsd lpconfig.xsd">
  <section name="proxy_default_values">
    <entry name="avpf" overwrite="true">1</entry>
    <entry name="dial_escape_plus" overwrite="true">0</entry>
    <entry name="publish" overwrite="true">1</entry>
    <entry name="quality_reporting_collector" overwrite="true">sip:voip-metrics@sip.linphone.org;transport=tls</entry>
    <entry name="quality_reporting_enabled" overwrite="true">1</entry>
    <entry name="quality_reporting_interval" overwrite="true">180</entry>
    <entry name="reg_expires" overwrite="true">600</entry>
    <entry name="reg_identity" overwrite="true">sip:?@sip.linphone.org</entry>
    <entry name="reg_proxy" overwrite="true">&lt;sip:sip.linphone.org;transport=tls&gt;</entry>
    <entry name="reg_sendregister" overwrite="true">1</entry>
    <entry name="nat_policy_ref" overwrite="true">nat_policy_default_values</entry>
    <entry name="realm" overwrite="true">sip.linphone.org</entry>
    <entry name="contact_parameters" overwrite="true">message-expires=604800</entry>
  </section>
  <section name="nat_policy_default_values">
    <entry name="stun_server" overwrite="true">stun.linphone.org</entry>
    <entry name="protocols" overwrite="true">stun,ice</entry>
  </section>
  <section name="misc">
    <entry name="file_transfer_server_url" overwrite="true">https://www.linphone.org:444/lft.php</entry>
  </section>
  <section name="sip">
    <entry name="rls_uri" overwrite="true">sips:rls@sip.linphone.org</entry>
  </section>
  <section name="assistant">
    <entry name="domain" overwrite="true">sip.linphone.org</entry>
    <entry name="algorithm" overwrite="true">SHA-256</entry>
    <entry name="password_max_length" overwrite="true">-1</entry>
    <entry name="password_min_length" overwrite="true">1</entry>
    <entry name="username_length" overwrite="true">-1</entry>
    <entry name="username_max_length" overwrite="true">64</entry>
    <entry name="username_min_length" overwrite="true">1</entry>
    <entry name="username_regex" overwrite="true">^[a-z0-9+_.\-]*$</entry>
    <entry name="xmlrpc_url" overwrite="true">https://subscribe.linphone.org:444/wizard.php</entry>
  </section>
</config>
